'----------------------------------------------------------------------------------------------------
'
' MCP23S17_DEBUG v1.00 - for testing the Microchip MCP23S17 I/O expander driver
'
' Copyright August 22, 2010 by William Henning
'
' This code is released under the terms of the MIT license, which is attached below.
'
'----------------------------------------------------------------------------------------------------

{{
┌──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┐
│                                                   TERMS OF USE: MIT License                                                  │
├──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
│Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    │
│files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    │
│modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software│
│is furnished to do so, subject to the following conditions:                                                                   │
│                                                                                                                              │
│The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.│
│                                                                                                                              │
│THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          │
│WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         │
│COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   │
│ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         │
└──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘
}}

CON

'----------------------------------------------------------------------------------------------------
' Constants defined for MCP23S17 use
'----------------------------------------------------------------------------------------------------

  _clkmode          = xtal1 + pll16x
  _xinfreq          = 5_000_000
   baud             = 38400

'----------------------------------------------------------------------------------------------------
' Objects needed for MCP23S17_DEBUG
'----------------------------------------------------------------------------------------------------

OBJ

  ser: "FullDuplexSerialPlus"   ' so we can talk to a terminal program over SerPlug
  pio: "MCP23S17"      ' I/O expander cog

VAR

  long command, addr, val

'----------------------------------------------------------------------------------------------------
' Variables defined for MCP23S17_DEBUG use
'----------------------------------------------------------------------------------------------------

  long mailbox[4]

  word joy0, joy1, buttons, nes

'----------------------------------------------------------------------------------------------------
' Main
'
'----------------------------------------------------------------------------------------------------

PUB main | ch

  ser.start(31,30,0,baud)
  pio.start(@mailbox) ' start up pioCOG
  waitcnt(cnt+80_000_000) ' give serial terminal window time

  repeat
    show(@menu)
    ch := ser.rx
    ser.tx(13)
    case ch
      "s": ' Show contents of all MCP23S17 registers, including input registers
       'repeat
        repeat ch from 0 to 21
          ser.tx(13)
          ser.hex(ch,2)
          ser.tx("=")
          ser.bin(pio.rdpbyte(0,ch),8)
        ser.tx(13)
        ser.hex(dira,8)
        ser.tx(13)
        ser.hex(ina,8)
        ser.tx(13)
      "w": ' Write a hex value into selected MCP23S17 register
        pio_write_pio_byte
      "r": ' LED rotation on both ports, programmed for output
        ' connect port pin to LED's through 270R-470R resistor
        ' anode of LED toward port pin, cathode to GND
        pio.wrpbyte(0,0,0) ' set GPA to output
        pio.wrpbyte(0,1,0) ' set GPB to output
        repeat 10
          repeat ch from 0 to 7
            pio.wrpbyte(0,$12,1<<ch) ' light selected LED on GPA
            pio.wrpbyte(0,$13,1<<ch) ' light selected LED on GPB
            waitcnt(cnt+40_000_000)
      other:
        ser.str(@unknown)

PUB pio_write_pio_byte|ad,v
  ad := get_addr
  v := get_value
  pio.wrpbyte(0,ad,v)
  ser.str(string("Wrote Byte $"))
  ser.hex(v,2)
  ser.str(string("to PIO register $"))
  ser.hex(ad,8)
  ser.tx(13)

PUB get_addr|t
  ser.str(string(13,"Enter HEX Address: "))
  t:= ser.GetHex
  ser.tx(13)
  return t

PUB get_value|t
  ser.str(string(13,"Enter HEX Value: "))
  t:= ser.GetHex
  ser.tx(13)
  return t

pub show(ptr)
  repeat until (byte[ptr]==0)
    ser.str(ptr)
    ser.tx(13)
    ptr+=strsize(ptr)+1

DAT

menu    byte 13,"MCP23S17 Debug Main Menu",13,0
        byte "s) show  MCP23S17 register contents",0
        byte "w) write MCP23S17 register to specified value",0
        byte "r) rotate LED's on GPA and GPB of the MCP23S17",0
        byte 13,"READY>",0,0

unknown byte "ERROR: Unknown Command",0

